from pydantic import BaseModel
from typing import List
import json

class Trigger(BaseModel):
    def execute(self):
        raise NotImplementedError("This method should be overridden by subclasses.")

class Action(BaseModel):
    _display_name: str = ""  # Add an internal variable to hold the display name
    _request_schema: BaseModel = None  # Placeholder for request schema
    _response_schema: BaseModel = None  # Placeholder for response schema

    @property
    def display_name(self) -> str:
        return self._display_name

    @display_name.setter
    def display_name(self, value: str):
        self._display_name = value  # Set the internal variable

    @property
    def request_schema(self) -> BaseModel:
        return self._request_schema

    @request_schema.setter
    def request_schema(self, value: BaseModel):
        self._request_schema = value

    @property
    def response_schema(self) -> BaseModel:
        return self._response_schema

    @response_schema.setter
    def response_schema(self, value: BaseModel):
        self._response_schema = value

    def execute(self, request_data: BaseModel, authorisation_data: dict) -> dict:
        raise NotImplementedError("This method should be overridden by subclasses.")
    
    @property
    def required_scopes(self) -> List[str]:
        return []


class Tool(BaseModel):
    def actions(self) -> List[Action]:
        raise NotImplementedError("This method should be overridden by subclasses.")
    
    def triggers(self) -> List[Trigger]:
        raise NotImplementedError("This method should be overridden by subclasses.")

    def execute_action(self, action_name: str, request_data: dict, authorisation_data: dict) -> dict:
        for action in self.actions():
            if action.__name__ == action_name:
                try:
                    request_schema = action().request_schema
                    req = request_schema.model_validate_json(json_data=json.dumps(request_data))
                    return action().execute(req, authorisation_data)
                except Exception as e:
                    return {"status": "failure", "details": str(e)}
        
    def json_schema(self):
        return {
            "Name": self.__class__.__name__,
            "Description": self.__doc__.strip() if self.__doc__ else None,
            "Actions": [
                {
                    "Id": action.__name__, 
                    "DisplayName": action().display_name,
                    "Description": action.__doc__.strip() if action.__doc__ else None,
                    "Signature": {
                        "Input": action().request_schema.model_json_schema(),
                        "Output": action().response_schema.model_json_schema()
                    }
                } for action in self.actions()
            ],
            "Triggers": [
                trigger.model_json_schema() for trigger in self.triggers()
            ],
        }

